/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <wintv.h>

#include <wintvchannel.h>

#include <kwm.h>
#include <kiconloader.h>
#include <klocale.h>

#define BID_ADD     0
#define BID_EDIT    1
#define BID_DELETE  2
#define BID_SL      3
#define BID_SC      4
#define BID_MUP     5
#define BID_MDOWN   6 

#ifdef myDEBUG
#define chanDEBUG
#endif

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

#define PID_MUTE  0
#define PID_SPLIT 1
#define PID_INPUT 2

const char * mixIconNames[] = { "audio", "bass", "treble", "midi", "audio",
				"unknown", "ext", "microphone", "cd", "recmon",
				"audio", "recmon", "recmon", "recmon", "ext",
				"ext", "ext" };
#define mixIconNamesSize 17

winSlider::winSlider( int _devID, int _min, int _max, 
		      int _valr, int _vall, 
		      bool _stereo, bool _input, 
		      bool _recsrc, bool _muteable, 
		      QPixmap * pix, QWidget * parent, const char * name )
  : QWidget( parent, name ), devID(_devID), min(_min), max(_max), lval(_valr), 
    rval(_vall), stereo(_stereo), input(_input), recsrc(_recsrc), muteable(_muteable)
{
  sleft = new QSlider( this );
  sleft->setMinimumSize( 10, 10 );
  sleft->setMaximumSize( 32767, 32767 );
  sleft->setRange( min, max );
  sleft->setSteps( 1, 10 );
  sleft->setValue( max-lval );
  sleft->setTracking( TRUE );
  sleft->setTickmarks( QSlider::Left );
  sleft->setTickInterval( 10 );

  sright = new QSlider( this );
  sright->setMinimumSize( 10, 10 );
  sright->setMaximumSize( 32767, 32767 );
  sright->setRange( min, max );
  sright->setSteps( 1, 10 );
  sright->setValue( max-rval );
  sright->setTracking( TRUE );
  sright->setTickmarks( QSlider::Right );
  sright->setTickInterval( 10 );

  icon= 0;
  if ( pix ) {
    icon = new QLabel( this );
    icon->setGeometry(0,0,20,20);
    icon->setPixmap( *pix );
  }
  
  connect(sright,SIGNAL(valueChanged(int)),this,SLOT(valueRChangedI(int)));
  connect(sleft,SIGNAL(valueChanged(int)),this,SLOT(valueLChangedI(int)));

  sPopup=new QPopupMenu;
  if ( muteable ) sid[PID_MUTE]=sPopup->insertItem(i18n("&Mute"),PID_MUTE);
  if ( stereo ) sid[PID_SPLIT]=sPopup->insertItem(i18n("Un&split"),PID_SPLIT);
  if ( input ) sid[PID_INPUT]=sPopup->insertItem(i18n("Rec&Source"),PID_INPUT);
  connect(sPopup,SIGNAL(activated(int)),this,SLOT(popupSLOT(int)));

  sright->installEventFilter( this );
  sleft->installEventFilter( this );

  setMinimumSize( 15 , 60 );

  if ( !stereo ) {
    splited= false;
    sright->hide();
  } else
    splited= true;
  
  muted=false;
  mm=true;
}

winSlider::~winSlider ()
{
  delete sleft;
  delete sright;
}

void winSlider::popupSLOT( int id )
{
  switch (id) {
  case PID_MUTE:
    mute(!muted);
    break;
  case PID_SPLIT:
    split(!splited);
    break;
  case PID_INPUT:
    crecsrc(!recsrc);
    break;
  default:
    warning("winSlider: popup");
  }
}

void winSlider::setLValue( int v )
{
  lval=v;
  sleft->setValue( max-v );
  if ( !splited ) {
    sright->setValue( max-v );
  }
}

void winSlider::setRValue( int v )
{
  rval=v;
  sright->setValue( max-v );
}

void winSlider::valueLChangedI( int v )
{
  lval=max-v;
  if ( !splited ) {
    rval=max-v;
    sright->setValue( v );
  }
  emit valueChanged( devID, rval, lval );
}

void winSlider::valueRChangedI( int v )
{
  rval=max-v;
  emit valueChanged( devID, rval, lval );
}

void winSlider::split( bool sp )
{
  if ( !stereo ) return;
  if ( splited != sp ) {
    splited= sp;
    if ( !splited ) {
      sPopup->changeItem(i18n("&Split"),sid[PID_SPLIT]);
      //sleft->setTickmarks( QSlider::Both );
      sright->hide();
      int val= (lval+rval)/2;
      sleft->setValue( max-val);
      sright->setValue( max-val );
    } else {
      sPopup->changeItem(i18n("Un&split"),sid[PID_SPLIT]);
      //sleft->setTickmarks( QSlider::Left );
      sright->setValue( max-rval );
      sright->show();
    }
    resizeEvent(NULL);
    emit splitChanged( devID, sp );
  }
}

void winSlider::crecsrc( bool on )
{
  if ( !input ) return;
  recsrc= on;
  if ( recsrc ) {
    sleft->setBackgroundColor( red ); 
    sright->setBackgroundColor( red ); 
  } else {
    if (muted) {
      sleft->setBackgroundColor( black ); 
      sright->setBackgroundColor( black ); 
    } else {
      sleft->setBackgroundColor( colorGroup().mid() ); 
      sright->setBackgroundColor( colorGroup().mid() ); 
    }
  }
  emit recsrcChanged( devID, on );
}

void winSlider::mute( bool on )
{
  if ( !muteable ) return;
  muted= on;
  if ( muted ) {
    if ( !recsrc ) {
      sleft->setBackgroundColor( black ); 
      sright->setBackgroundColor( black ); 
    }
    sPopup->changeItem(i18n("&Unmute"),sid[PID_MUTE]);
  } else {
    if ( !recsrc ) {
      sleft->setBackgroundColor( colorGroup().mid() ); 
      sright->setBackgroundColor( colorGroup().mid() ); 
    }
    sPopup->changeItem(i18n("&Mute"),sid[PID_MUTE]);
  }
  emit muteChanged( devID, on );
}

void winSlider::setEnabled( bool on )
{
  mm=on;
  sleft->setEnabled(on);
  sright->setEnabled(on);
}

void winSlider::resizeEvent ( QResizeEvent *  )
{
  if ( icon ) {
    icon->setGeometry( width()/2 - 10, 0, 20, 20 );
    yoff= 25;
  } else {
    yoff= 0;
  }
  if (splited) {
    sleft->setGeometry( 0, yoff, width()/2-1, height()-yoff );
    sright->setGeometry( sleft->width(), yoff, width()/2-1, height()-yoff );
  } else {
    sleft->setGeometry( 0, yoff, width()-2, height()-yoff );
  }
}

bool winSlider::eventFilter( QObject *o, QEvent *e )
{
  // Lets see, if we have a "Right mouse button press"
  if (e->type() == Event_MouseButtonPress) {
    QMouseEvent *qme = (QMouseEvent*)e;
    if (qme->button() == RightButton) {
      sPopup->popup(QCursor::pos()/*mapToGlobal(qme->pos())*/);
      return true;
    }
  }
  return false;
}

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

winMixer::winMixer( rcfile * _rc,
		    QWidget * parent, const char * name )
  : rc(_rc)
{
  KIconLoader *loader = kapp->getIconLoader();
  QPixmap pm = loader->loadIcon("kwintv_48x48.xpm");
  setIcon(pm);
  setIconText(i18n("kWinTV Mixer"));
  setCaption(i18n("kWinTV Mixer"));

  config=kapp->getConfig();
  config->setGroup("CONFIG__mixer");

  // init sound mixer
  kmix= new mixer((const char *)rc->mixerDeviceName);

  // set the kwintv mixing device
  // try line in, then other input devices
  if ( rc->mixerDevice == NO_MDEV ) {
    rc->mixerDevice= SOUND_MIXER_LINE;
    if ( ! kmix->isInputDevice(rc->mixerDevice) ) {
      int i;
      for ( i=0; i < SOUND_MIXER_NRDEVICES; i++ )
	if ( kmix->isInputDevice(i) ) {
	  rc->mixerDevice= i;
	  break;
	}
      if ( i == SOUND_MIXER_NRDEVICES ) {
	warning("winslider: was unable to find any sound mixer input device!");
	rc->mixerDevice= NO_MDEV;
      }
    }
  }

  // fetch all device names
  int devNr;
  char *devName;
  winSlider *ws;
  slidlist= new QIntDict<winSlider>;
  devName=kmix->getFirstDeviceName(&devNr);

  if ( !devName ) {
    warning("winmixer: no devices found!");
  } else while ( devName ) {
    if ( devNr == rc->mixerDevice ) {
      pm = loader->loadIcon("kwintv_24x24.xpm");
    } else {
      if ( devNr < mixIconNamesSize ) {
	pm = loader->loadIcon("mix_"+QString(mixIconNames[devNr])+".xpm");
      } else {
	pm = loader->loadIcon("mix_unknown.xpm");
      }
    }

    ws= new winSlider(devNr,0,MIXER_MAX_VALUE,
		      kmix->getVolume(devNr,VOL_RIGHT),
		      kmix->getVolume(devNr,VOL_LEFT),
		      kmix->isStereoDevice(devNr),
		      kmix->isInputDevice(devNr),
		      kmix->isRecSource(devNr),true,&pm,this,
		      (const char *)QString(devName).lower());
    //ws->mute(true);
    ws->crecsrc(kmix->isRecSource(devNr));
    ws->split(config->readNumEntry("split_"+QString(devName).lower(),0));

    slidlist->insert(devNr,ws);

    if ( devNr == rc->mixerDevice ) {
      if ( !rc->use_master_volume ) {
#ifdef chanDEBUG
	debug("winmixer: set old volume: %d,%d",rc->volume_right,rc->volume_left);
#endif
//  	ws->setRValue(MIXER_MAX_VALUE-rc->volume_right);
//  	ws->setLValue(MIXER_MAX_VALUE-rc->volume_left);
	ws->setRValue(rc->volume_right);
	ws->setLValue(rc->volume_left);
	kmix->setStereoVolume(devNr,rc->volume_right,rc->volume_left);
      } else { // use master vol
	rc->volume_right=kmix->getVolume(devNr,VOL_RIGHT);
	rc->volume_left=kmix->getVolume(devNr,VOL_LEFT);
#ifdef chanDEBUG
	debug("winmixer: set master volume: %d,%d",rc->volume_right,rc->volume_left);
#endif
      }
      QToolTip::add(ws,"kWinTV ("+QString(devName)+")");    
    } else {
      QToolTip::add(ws,devName);
    }

    connect(ws,SIGNAL(valueChanged(int,int,int)),this,SLOT(changeVol(int,int,int)));
    connect(ws,SIGNAL(muteChanged(int,bool)),this,SLOT(changeMute(int,bool)));
    connect(ws,SIGNAL(splitChanged(int,bool)),this,SLOT(changeSize(int,bool)));
    connect(ws,SIGNAL(recsrcChanged(int,bool)),this,SLOT(changeRecsrc(int,bool)));

    devName=kmix->getNextDeviceName(&devNr);
  }

  // setMinimumSize( *slidlist->count() , 20+10+100 );
  resize( 45*slidlist->count(), 20+10+100 );
  changeSize(0,false);

  if (rc->ol_restore) {
    QPoint ppos= config->readPointEntry("xypos");
    //move(ppos);
    setGeometry(ppos.x(),ppos.y(),width(),height());
  }
}

winMixer::~winMixer()
{
#ifdef chanDEBUG
  debug("winmixer: ~winMixer()");
#endif
  config->setGroup("CONFIG__mixer");
  config->writeEntry("xypos",pos());
  
  QIntDictIterator<winSlider> it( *slidlist );
  while (it.current()) {
#ifdef chanDEBUG_A
    debug("winmixer: save %s",it.current()->name());
#endif
    config->writeEntry("split_"+QString(it.current()->name())
		       ,it.current()->isSplited()?1:0);
    ++it;
  }

  delete slidlist;
  delete kmix;

  //config->sync();
}

void winMixer::extVolSet( int rval, int lval )
{
  if ( rc->volume_right==rval && rc->volume_left == lval) return;
  rc->volume_right=rval;
  rc->volume_left=lval;
  (*slidlist)[rc->mixerDevice]->setRValue(rval);
  (*slidlist)[rc->mixerDevice]->setLValue(lval);
}
void winMixer::showMixer()
{
  isVisible()?hide():show();
}

void winMixer::changeVol( int devID, int rval, int lval )
{
  kmix->setStereoVolume(devID,rval,lval);
  if ( devID == rc->mixerDevice ) {
    rc->volume_right=rval;
    rc->volume_left=lval;
  }
  emit volChanged(rval,lval);
}

void winMixer::changeRecsrc( int devID, bool on )
{
  kmix->setRecSource(devID,on);
}

void winMixer::extVolMute( bool on ) // on==true -> mute ( -> rc->sound_on=false )
{
#ifdef chanDEBUG
  debug("winMixer: extVolMute %d",on);
#endif
  if ( on == !rc->sound_on ) return;
  rc->sound_on=!on;
  (*slidlist)[rc->mixerDevice]->mute(on);
}
void winMixer::extMuteRequest( bool on )
{
#ifdef chanDEBUG
  debug("winMixer: extMuteRequest %d",on);
#endif
  if ( !rc->sound_on ) return;
  if ( on ) {
    kmix->setStereoVolume(rc->mixerDevice,0,0);
  } else {
    kmix->setStereoVolume(rc->mixerDevice,
			  (*slidlist)[rc->mixerDevice]->rVol(),
			  (*slidlist)[rc->mixerDevice]->lVol());
  }
}
void winMixer::changeMute( int devID, bool on ) // on==true -> mute ( -> rc->sound_on=false )
{
  if ( on ) {
    kmix->setStereoVolume(devID,0,0);
  } else {
    kmix->setStereoVolume(devID,(*slidlist)[devID]->rVol(),(*slidlist)[devID]->lVol());
  }
  if ( devID == rc->mixerDevice ) {
    if ( on == !rc->sound_on ) return;
    rc->sound_on= !on;
    emit volMuteChanged( rc->sound_on );
  }
}

void winMixer::changeSize( int id, bool on )
{
#ifdef chanDEBUG
  debug("winMixer: changeSize");
#endif 
  xoff=5;
  QIntDictIterator<winSlider> it( *slidlist );
  while (it.current()) {
    xoff= 3 + xoff + (it.current()->isSplited()?46:24);
    ++it;
  }
  resize(xoff,height());
  setFixedSize(xoff,height());
}

void winMixer::resizeEvent ( QResizeEvent * )
{
  xoff=5;
  QIntDictIterator<winSlider> it( *slidlist );
  while (it.current()) {
    it.current()->setGeometry(xoff,5,it.current()->isSplited()?46:24,height()-10);
    xoff= 3 + xoff + (it.current()->isSplited()?46:24);
    ++it;
  }
}

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

winTVChannelWidget::winTVChannelWidget(rcfile *_rc,
				       QWidget* parent, const char* name)
  : KTopLevelWidget( name ),rc(_rc)
{
#ifdef chanDEBUG
  debug("channel: begin of init");
#endif

  KWM::setIcon(winId(), kapp->getIcon());
  KWM::setMiniIcon(winId(), kapp->getMiniIcon());

  setCaption((const char*)(QString(i18n("Program Browser"))));
  // set icon

  KIconLoader *loader = kapp->getIconLoader();

  pm = loader->loadIcon("kwintv_logo.xpm");
  setIcon(pm);
  setIconText(i18n("Program Browser"));

  // toolbar
  toolbar = new KToolBar(this,"-",26);
  toolbar->setBarPos(KToolBar::Top);
  addToolBar(toolbar);

  ch_add_xpm     = loader->loadIcon("filenew.xpm");
  ch_edit_xpm    = loader->loadIcon("editpaste.xpm");
  ch_delete_xpm  = loader->loadIcon("editcut.xpm");
  mup_xpm        = loader->loadIcon("up.xpm");
  mdown_xpm      = loader->loadIcon("down.xpm");
  cs_xpm         = loader->loadIcon("delete.xpm");
  
  toolbar->insertButton(ch_add_xpm,BID_ADD,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("add new program"));
  toolbar->insertButton(ch_edit_xpm,BID_EDIT,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("edit this program"));
  toolbar->insertButton(ch_delete_xpm,BID_DELETE,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("delete this program"));
#if 1
  toolbar->insertSeparator();
  
  toolbar->insertButton(mup_xpm,BID_MUP,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("move up"));
  toolbar->insertButton(mdown_xpm,BID_MDOWN,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("move down"));
#endif
  toolbar->insertSeparator();

  toolbar->insertButton(cs_xpm,BID_SC,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("close program browser"));

  channelbox= new winWidget(this);
  setView(channelbox);

  connect(channelbox,SIGNAL(highlighted(int)),this,SLOT(channelSelectedEmitSLOT(int)));
  connect(channelbox,SIGNAL(channelEditSelectSIGNAL()),this,SLOT(channelEditEmitSLOT()));
  connect(channelbox,SIGNAL(moveItemSIGNAL( int, int )),this,SLOT(moveItemSLOT( int, int )));

  config=kapp->getConfig();
  
  setMinimumSize( 180,100 );

  if (rc->ol_restore) {
    config->setGroup("CONFIG__browser");
    QPoint pp= config->readPointEntry("xypos");
    //move(pp);
    QSize pso(210,260);
    QSize ps= config->readSizeEntry("wsize",&pso);
    //    resize(ps);
    setGeometry(pp.x(),pp.y(),ps.width(),ps.height());
  } else {
    resize( 210,260 );
  }
}

/* ------------------------------------------------------------------------------*/

winTVChannelWidget::~winTVChannelWidget()
{
  config->setGroup("CONFIG__browser");
  config->writeEntry("xypos",pos());
  config->writeEntry("wsize",size());
  //config->sync();
  delete toolbar;
  delete channelbox;
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::newChannelListSLOT( QList<channel> *clist )
{
  if ( clist->isEmpty() ) return;

  clistp= clist;

  int id= clist->at();

  inserting=TRUE;
  channelbox->clear();
  
  channel *c= clist->first();
  while ( c ) {
    c->id=clist->at();
    channelbox->insertItem( c, inserting );
    c= clist->next();
  }
  inserting=FALSE;

  clist->at(id);

#ifdef chanDEBUG
  debug("channel: channel list insertion done %d, %d",clist->count(),clist->at());
#endif
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::moveItemSLOT( int from, int to )
{
  if ( from == to || from == -1 || to == -1 || from == clistp->count() || to == clistp->count() )
    return;

#ifdef chanDEBUG
  debug("channel: move %d -> %d",from,to);
#endif

  inserting=TRUE;

  if ( from > to ) {
    channel *cfrom= new channel(*clistp->at(from));
    channel *cto= clistp->at(to);
    
    clistp->remove( from );
    clistp->insert( clistp->find(cto), cfrom );
  } else { // from < to 
    channel *cfrom= new channel(*clistp->at(from));
    channel *cto= clistp->at(to+1);
    
    clistp->remove( from );
    if ( cto == NULL ) 
      clistp->append(cfrom );
    else 
      clistp->insert( clistp->find(cto), cfrom );
  }
  inserting=FALSE;

  newChannelListSLOT( clistp );

  clistp->at(to);
  channelbox->setCurrentItem( clistp->at() );
  channelbox->centerCurrentItem();
  emit channelSelectedSIGNAL();
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::activeChannelChangedSLOT()
{
  if (inserting) return;

#ifdef chanDEBUG
  debug("channel: activeChannelChangedSLOT");
#endif

  channelbox->setCurrentItem( clistp->at() );
  channelbox->centerCurrentItem();
  channelbox->repaintItem();
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::toggleVisibilitySLOT( QPoint *to )
{
  if (isVisible()) hide();
  else {
    if ( to ) {
#ifdef chanDEBUG
      debug("channel: setGeometry %d,%d,%d,%d",to->x(), to->y(), width(), height());
#endif
      setGeometry( to->x(), to->y(), width(), height() );
    }
    show();
  }
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::channelSelectedEmitSLOT(int index)
{
  if (inserting) return;

#ifdef chanDEBUG
  debug("channel: channelSelectedEmitSLOT");
#endif

  if ( index != clistp->at() ) {
    clistp->at( index );
    emit channelSelectedSIGNAL();
  }
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::channelEditEmitSLOT()
{
  if (inserting) return;

#ifdef chanDEBUG
  debug("channel: channelEditEmitSLOT");
#endif

  emit editChannelSIGNAL();
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::buttonsSLOT(int button)
{
  switch (button) {
  case BID_SC:
    toggleVisibilitySLOT( NULL );
    break;
  case BID_ADD:
    emit addChannelSIGNAL();
    break;
  case BID_EDIT:
    emit editChannelSIGNAL();
    break;
  case BID_DELETE:
    emit deleteChannelSIGNAL();
    break;
  case BID_MUP:
    //    emit copyChannelSIGNAL();
    moveItemSLOT(clistp->at(),clistp->at()-1);
    break;
  case BID_MDOWN:
    //    emit pasteChannelSIGNAL();
    moveItemSLOT(clistp->at(),clistp->at()+1);
    break;
  default:
    // this should never happen...
    fatal("main: nonexisting button number");
  }
}


/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::setPopup(QPopupMenu *cp)
{
  channelbox->setCPopup(cp);
}

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

#define Inherited QWidget

winWidget::winWidget(QWidget* parent, const char* name)
  : Inherited( parent, name, 503808 )
{
#ifdef chanDEBUG
  debug("win: init start");
#endif
  channelList = new winListBox( this );
  channelList->setGeometry( 5, 5, 140, 148 );
  channelList->setMinimumSize( 10, 10 );
  channelList->setMaximumSize( 32767, 32767 );

  channelList->setFrameStyle( 51 );
  channelList->setLineWidth( 2 );
  channelList->setMultiSelection ( false );
  channelList->setAllColumnsShowFocus( true );
  channelList->setTreeStepSize( 0 ); 
  channelList->setRootIsDecorated( false );

  channelList->addColumn(i18n("Nr."));
  channelList->setColumnWidth(0,30);
  channelList->addColumn(i18n("Name"));
  channelList->setColumnWidth(1,90);
  channelList->addColumn(i18n("Channel"));
  channelList->setColumnWidth(2,60);

  QToolTip::add(channelList,i18n("Programs"));

  connect(channelList,SIGNAL(selectionChanged ( QListViewItem * ) ),
	  this,SLOT(highlightedSLOT(QListViewItem *)));
  connect(channelList,SIGNAL( doubleClicked ( QListViewItem * ) ),
	  this,SLOT(channelEditSelectSLOT(QListViewItem *)));
  connect(channelList,SIGNAL( returnPressed ( QListViewItem * ) ),
	  this,SLOT(channelEditSelectSLOT(QListViewItem *)));

  connect(channelList,SIGNAL(moveItemSIGNAL( QListViewItem *, QListViewItem * )),
	  this, SLOT(moveItemSLOT( QListViewItem *, QListViewItem * )));
  

  channelList->setFocus();
}

// =============================================================================== 

winWidget::~winWidget()
{

}

// =============================================================================== 

void winWidget::channelEditSelectSLOT(QListViewItem *item)
{
#ifdef chanDEBUG
  debug("win:  channelEditSelectSLOT");
#endif
  emit channelEditSelectSIGNAL();
}

// =============================================================================== 

void winWidget::highlightedSLOT(QListViewItem * item)
{
  if (inserting) return;

  int id= aChannelList.find( item );
  
#ifdef chanDEBUG
  debug("win:  highlightedSLOT %d, %s",id,item->text(2));
#endif

  emit highlighted(id);
}

// =============================================================================== 

void winWidget::insertItem ( const channel* info, bool ins)  
{
  inserting=ins;

  QString id;
  id.sprintf("%2d",info->id);

  QListViewItem *lvi= new QListViewItem( channelList, (const char*)id, info->name, info->channelId );
  aChannelList.append(lvi);

  inserting=false;

#ifdef chanDEBUG
  //debug("win:  insertItem %s, %s, %s, %s, %d",(const char*)id,info->name,(const char*)freq,info->channelId,inserting);
#endif
}


// =============================================================================== 

void winWidget::removeItem ( int index ) 
{
#ifdef chanDEBUG
  debug("win: removeItem");
#endif
  QListViewItem *lvi= aChannelList.at( index );
  aChannelList.remove();
  delete lvi;
  if ( aChannelList.count() ) {
    channelList->setSelected ( aChannelList.current(), true );
    channelList->setCurrentItem ( aChannelList.current() ); 
    channelList->ensureItemVisible(channelList->currentItem());
    //channelList->setSorting( 0 , true );
  }
}

// =============================================================================== 

void winWidget::changeItem ( const char * text, int index ) 
{
#ifdef chanDEBUG
  debug("win: changeItem %d",index);
#endif
  QListViewItem *lvi= aChannelList.at( index );
  lvi->setText( 2, text );
  channelList->repaintItem( lvi );
}

// =============================================================================== 

void winWidget::setCurrentItem ( int index )  
{
#ifdef chanDEBUG
  debug("win: setCurrentItem %d",index);
#endif
  channelList->setSelected ( aChannelList.at( index ), true );
  channelList->setCurrentItem ( aChannelList.at( index ) );  
}

// =============================================================================== 

void winWidget::centerCurrentItem ()  
{
  channelList->ensureItemVisible( aChannelList.current() );
  channelList->repaintItem( aChannelList.current() ); 
}

// =============================================================================== 

void winWidget::repaintItem ()  
{
  channelList->repaintItem( aChannelList.current() );
}

// =============================================================================== 

void winWidget::clear()  
{
#ifdef chanDEBUG
  debug("win: clear");
#endif

  inserting=true;

  channelList->clear();
  if ( !aChannelList.isEmpty() ) aChannelList.clear();
  channelList->setSorting( 0 , true );

  inserting=false;
}

// ================================"=============================================== 

void winWidget::moveItemSLOT( QListViewItem * fromItem, QListViewItem *toItem)
{
#ifdef chanDEBUG
  debug("win: move item %s to pos %s",fromItem->text(1),toItem->text(0));
#endif
  if ( !aChannelList.isEmpty() ) {
    QString from( fromItem->text(0) );
    QString to( toItem->text(0));

    emit moveItemSIGNAL( from.toInt(), to.toInt() ); 
  }
  
}

// =============================================================================== 
void winWidget::resizeEvent( QResizeEvent * re )
{
  channelList->setGeometry( 5, 5, width()-10, height()-10 );
}

// =============================================================================== 
// =============================================================================== 
// =============================================================================== 


void winListBox::mouseReleaseEvent ( QMouseEvent * me )
{
  if (me->button() == RightButton) {
    cPopup->popup(QCursor::pos()/*mapToGlobal(qme->pos())*/);
    return;
  }
#ifdef DRAGDROP
  if ( !pressedButton ) QListView::mouseReleaseEvent( me );

  switch (me->button()) {
  case LeftButton:
#ifdef chanDEBUG
    debug("channelListBox: mouseReleaseEvent, LeftButton released");
#endif
    if ( itemWasMoved ) {
#ifdef chanDEBUG
      debug("channelListBox: mouseReleaseEvent, item moved, LeftButton released");
#endif
      QListViewItem *targetItem= currentItem();
      if ( targetItem && (toMoveItem != targetItem) ) {
#ifdef chanDEBUG
	debug("channelListBox: mouseReleaseEvent, moving item to %s",(const char *)targetItem->text(1));
#endif
	emit moveItemSIGNAL( toMoveItem, targetItem );
      } else {
	setCurrentItem( currentItem() );
      }
      itemWasMoved = false;
      pressedButton= 0;
    } else {
#ifdef chanDEBUG
      debug("channelListBox: mouseReleaseEvent, item not moved, LeftButton released");
#endif
      itemWasMoved = false;
      pressedButton= 0;
    }
  } // switch
#endif
}

// =============================================================================== 
#ifdef DRAGDROP
void winListBox::mousePressEvent ( QMouseEvent * me )
{
  QListView::mousePressEvent ( me );
  
  if (me->button() == LeftButton) {
#ifdef chanDEBUG
    debug("winListBox: mousePressEvent, LeftButton");
#endif
    pressedButton = LeftButton;
    toMoveItem= itemAt( me->pos() );
  } else 
    pressedButton = 0;
  
}
#endif
// =============================================================================== 

void winListBox::mouseMoveEvent ( QMouseEvent * me )
{
#ifdef DRAGDROP
  if ( !pressedButton ) QListView::mouseMoveEvent( me );

  if (pressedButton == LeftButton) {

#ifdef chanDEBUG
    debug("winListBox: mouseMoveEvent, LeftButton");
#endif

    oldY= me->y();

    if ( geometry().contains( me->pos() ) && (itemAt( me->pos() ) != NULL) ) {
      lastVisItem=itemAt( me->pos() );
      setCurrentItem( lastVisItem );
      /*
      ensureItemVisible( lastVisItem );

      if ( itemAt( me->pos() ) != lastVisItem ) {
	QPoint pos( mapToGlobal(me->pos()).x() , itemPos( lastVisItem ));
	QCursor::setPos( mapFromGlobal(pos) );
      }
      */
      itemWasMoved=true;
      if ( !cursorChanged ) {
	// change mouse cursor
	cursorChanged= true;
      }
#ifdef chanDEBUG
      //debug("winListBox: mouseMoveEvent: move");
#endif
    } else {
#ifdef chanDEBUG
      debug("winListBox: mouseMoveEvent: exit");
#endif
      itemWasMoved=false;
      setCurrentItem( toMoveItem );
      //ensureItemVisible ( currentItem() );
      pressedButton=0;
      cursorChanged= false;
    }
  }
#endif
}

// =============================================================================== 
// =============================================================================== 
// =============================================================================== 
// never change the next line!
#include "wintvchannel.moc"

// local Variables: ***
// compile-command-1: "make kwintvg" ***
// compile-command-2: "make" ***
// End: ***
